@@ -2,11 +2,11 @@ use core::fmt;
 use std::ops::Neg;
 
 /// Define a time in milliseconds
-#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct TimePoint(i64);
 
 impl TimePoint {
-    /// Create a `TimePoint` from miliseconds
+    /// Create a `TimePoint` from milliseconds
     #[must_use]
     pub const fn from_msecs(time: i64) -> Self {
         Self(time)
@@ -17,7 +17,7 @@ impl TimePoint {
     /// # Panics
     ///
     /// Will panics if the `seconds` value fill as parameter is to big to be store as
-    /// millisecond in a i64.
+    /// millisecond in a [`i64`].
     #[must_use]
     pub fn from_secs(seconds: f64) -> Self {
         let msecs = cast::i64(seconds * 1000.0).unwrap();
@@ -91,7 +91,7 @@ mod tests {
     }
 
     #[test]
-    fn time_point_creation_with_too_mutch_decimals() {
+    fn time_point_creation_with_too_much_decimals() {
         assert_eq!(TimePoint::from_msecs(265), TimePoint::from_secs(0.265_579));
         assert_eq!(TimePoint(142), TimePoint::from_secs(0.142_75));
     }
