@@ -4,8 +4,17 @@
 //!
 //! [subs]: http://sam.zoy.org/writings/dvd/subtitles/
 
-use cast;
-use image::{ImageBuffer, Rgba, RgbaImage};
+use super::{decoder::VobSubDecoder, img::VobSubIndexedImage, mpeg2::ps, VobSubError};
+use crate::{
+    content::{Area, AreaValues},
+    time::TimeSpan,
+    util::BytesFormatter,
+    vobsub::{
+        img::{VobSubRleImage, VobSubRleImageData},
+        IResultExt,
+    },
+};
+use iter_fixed::IntoIteratorFixed;
 use log::{trace, warn};
 use nom::{
     bits::{bits, complete::take as take_bits},
@@ -18,25 +27,9 @@ use nom::{
     sequence::{preceded, Tuple},
     IResult,
 };
-use std::{cmp::Ordering, fmt};
+use std::{cmp::Ordering, fmt::Debug, marker::PhantomData};
 use thiserror::Error;
 
-use super::{img::decompress, mpeg2::ps, Palette, VobSubError};
-use crate::{
-    content::{Area, AreaValues},
-    util::BytesFormatter,
-    vobsub::IResultExt,
-};
-
-/// The default time between two adjacent subtitles if no end time is
-/// provided.  This is chosen to be a value that's usually representable in
-/// `SRT` format, barring rounding errors.
-const DEFAULT_SUBTITLE_SPACING: f64 = 0.001;
-
-/// The default length of a subtitle if no end time is provided and no
-/// subtitle follows immediately after.
-const DEFAULT_SUBTITLE_LENGTH: f64 = 5.0;
-
 /// Parse four 4-bit palette entries.
 fn palette_entries(input: &[u8]) -> IResult<&[u8], [u8; 4]> {
     let (input, vec) = bits(count(
@@ -64,7 +57,7 @@ fn area(input: &[u8]) -> IResult<&[u8], AreaValues> {
     })(input)
 }
 
-/// Parse a pair of 16-bit RLE offsets.
+/// Parse a pair of 16-bit `RLE` offsets.
 fn rle_offsets(input: &[u8]) -> IResult<&[u8], [u16; 2]> {
     let (input, vec) = bits(count(
         take_bits::<_, _, _, nom::error::Error<(&[u8], usize)>>(16u16),
@@ -167,110 +160,6 @@ fn control_sequence(input: &[u8]) -> IResult<&[u8], ControlSequence> {
     ))
 }
 
-/// A single subtitle.
-#[derive(Clone, PartialEq)]
-pub struct Subtitle {
-    /// Start time of subtitle, in seconds.
-    start_time: f64,
-    /// End time of subtitle, in seconds.  This may be missing from certain
-    /// subtitles.
-    end_time: Option<f64>,
-    /// Should this subtitle be shown even when subtitles are off?
-    force: bool,
-    /// Area coordinates at which to display the subtitle.
-    area: Area,
-    /// Map each of the 4 colors in this subtitle to a 4-bit palette.
-    palette: [u8; 4],
-    /// Map each of the 4 colors in this subtitle to 4 bits of alpha
-    /// channel data.
-    alpha: [u8; 4],
-    /// Our decompressed image, stored with 2 bits per byte in row-major
-    /// order, that can be used as indices into `palette` and `alpha`.
-    raw_image: Vec<u8>,
-}
-
-impl Subtitle {
-    /// Start time of subtitle, in seconds.
-    #[must_use]
-    pub const fn start_time(&self) -> f64 {
-        self.start_time
-    }
-
-    /// End time of subtitle, in seconds. This may be missing from certain
-    /// subtitles.
-    /// # Panics
-    /// Will panic if `end_time` is not set. As it should be set before returning subtitle.
-    /// If happened `end_time` is called to soon, or a change has broken the intended operation.
-    #[must_use]
-    pub fn end_time(&self) -> f64 {
-        self.end_time
-            .expect("end time should have been set before returning subtitle")
-    }
-
-    /// Should this subtitle be shown even when subtitles are off?
-    #[must_use]
-    pub const fn force(&self) -> bool {
-        self.force
-    }
-
-    /// Coordinates at which to display the subtitle.
-    #[must_use]
-    pub const fn area(&self) -> &Area {
-        &self.area
-    }
-
-    /// Map each of the 4 colors in this subtitle to a 4-bit palette.
-    #[must_use]
-    pub const fn palette(&self) -> &[u8; 4] {
-        &self.palette
-    }
-
-    /// Map each of the 4 colors in this subtitle to 4 bits of alpha
-    /// channel data.
-    #[must_use]
-    pub const fn alpha(&self) -> &[u8; 4] {
-        &self.alpha
-    }
-
-    /// Our decompressed image, stored with 2 bits per byte in row-major
-    /// order, that can be used as indices into `palette` and `alpha`.
-    #[must_use]
-    pub fn raw_image(&self) -> &[u8] {
-        &self.raw_image
-    }
-
-    /// Decompress to subtitle to an RBGA image.
-    #[must_use]
-    pub fn to_image(&self, palette: &Palette) -> RgbaImage {
-        let width = cast::u32(self.area.width());
-        let height = cast::u32(self.area.height());
-        ImageBuffer::from_fn(width, height, |x, y| {
-            let offset = cast::usize(y * width + x);
-            // We need to subtract the raw index from 3 to get the same
-            // results as everybody else.  I found this by inspecting the
-            // Handbrake subtitle decoding routines.
-            let px = cast::usize(3 - self.raw_image[offset]);
-            let rgb = palette[cast::usize(self.palette[px])].0;
-            let a = self.alpha[px];
-            let aa = a << 4 | a;
-            Rgba([rgb[0], rgb[1], rgb[2], aa])
-        })
-    }
-}
-
-impl fmt::Debug for Subtitle {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        fmt.debug_struct("Subtitle")
-            .field("start_time", &self.start_time)
-            .field("end_time", &self.end_time)
-            .field("force", &self.force)
-            .field("area", &self.area)
-            .field("palette", &self.palette)
-            .field("alpha", &self.alpha)
-            .finish_non_exhaustive()
-    }
-}
-
 /// Parse a single `u16` value from a buffer.  We don't use `nom` for this
 /// because it has an inconvenient error type.
 fn parse_be_u16_as_usize(buff: &[u8]) -> Result<(&[u8], usize), VobSubError> {
@@ -300,13 +189,17 @@ pub enum ErrorMissing {
     #[error("no alpha palette")]
     AlphaPalette,
 
-    /// No RLE offsets
+    /// No `RLE` offsets
     #[error("no RLE offsets")]
     RleOffset,
 }
 
 /// Parse a subtitle.
-fn subtitle(raw_data: &[u8], base_time: f64) -> Result<Subtitle, VobSubError> {
+fn subtitle<'a, D, T>(raw_data: &'a [u8], base_time: f64) -> Result<T, VobSubError>
+where
+    T: Debug,
+    D: VobSubDecoder<'a, Output = T>,
+{
     // This parser is somewhat non-standard, because we need to work with
     // explicit offsets into `packet` in several places.
 
@@ -377,7 +270,7 @@ fn subtitle(raw_data: &[u8], base_time: f64) -> Result<Subtitle, VobSubError> {
 
         // Figure out where to look for the next control sequence,
         // if any.
-        let next_control_offset = cast::usize(control.next);
+        let next_control_offset = usize::from(control.next);
         match control_offset.cmp(&next_control_offset) {
             Ordering::Greater => {
                 return Err(VobSubError::ControlOffsetWentBackwards);
@@ -400,43 +293,15 @@ fn subtitle(raw_data: &[u8], base_time: f64) -> Result<Subtitle, VobSubError> {
     let rle_offsets = rle_offsets.ok_or(ErrorMissing::RleOffset)?;
 
     // Decompress our image.
-    //
-    // We know the starting points of each set of scan lines, but we don't
-    // really know where they end, because encoders like to reuse bytes
-    // that they're already using for something else.  For example, the
-    // last few bytes of the first set of scan lines may overlap with the
-    // first bytes of the second set of scanlines, and the last bytes of
-    // the second set of scan lines may overlap with the start of the
-    // control sequence.  For now, we limit it to the first two bytes of
-    // the control packet, which are usually `[0x00, 0x00]`.  (We might
-    // actually want to remove `end` entirely here and allow the scan lines
-    // to go to the end of the packet, but I've never seen that in
-    // practice.)
-    let start_0 = cast::usize(rle_offsets[0]);
-    let start_1 = cast::usize(rle_offsets[1]);
-    let end = cast::usize(initial_control_offset + 2);
-    if start_0 > start_1 || start_1 > end {
-        return Err(VobSubError::InvalidScanLineOffsets {
-            start_0,
-            start_1,
-            end,
-        });
-    }
-    let image = decompress(
-        area.size(),
-        [&raw_data[start_0..end], &raw_data[start_1..end]],
-    )?;
+    let end = initial_control_offset + 2;
+    // reverse palette & alpha once for all
+    let palette = palette.into_iter_fixed().rev().collect();
+    let alpha = alpha.into_iter_fixed().rev().collect();
+    let image_data = VobSubRleImageData::new(raw_data, rle_offsets, end)?;
+    let rle_image = VobSubRleImage::new(area, palette, alpha, image_data);
 
     // Return our parsed subtitle.
-    let result = Subtitle {
-        start_time,
-        end_time,
-        force,
-        area,
-        palette,
-        alpha,
-        raw_image: image,
-    };
+    let result = D::from_data(start_time, end_time, force, rle_image);
     trace!("Parsed subtitle: {:?}", &result);
     Ok(result)
 }
@@ -457,17 +322,27 @@ macro_rules! try_iter {
 /// An internal iterator over subtitles.  These subtitles may not have a
 /// valid `end_time`, so we'll try to fix them up before letting the user
 /// see them.
-struct SubtitlesInternal<'a> {
+pub struct VobsubParser<'a, Decoder> {
     pes_packets: ps::PesPackets<'a>,
+    phantom_data: PhantomData<Decoder>,
 }
 
-impl<'a> Iterator for SubtitlesInternal<'a> {
-    type Item = Result<Subtitle, VobSubError>;
+impl<'a, Decoder> VobsubParser<'a, Decoder> {
+    /// To parse a `vobsub` (.sub) file content.
+    /// Return an iterator over the subtitles in this data stream.
+    #[must_use]
+    pub const fn new(input: &'a [u8]) -> Self {
+        Self {
+            pes_packets: ps::pes_packets(input),
+            phantom_data: PhantomData,
+        }
+    }
 
-    fn next(&mut self) -> Option<Self::Item> {
-        profiling::scope!("SubtitlesInternal next");
+    // Read all pes_packets needed to parse a subtitle.
+    fn next_sub_packet(&mut self) -> Option<Result<(f64, Vec<u8>), VobSubError>> {
+        profiling::scope!("VobsubParser next_sub_packet");
 
-        // Get the PES packet containing the first chunk of our subtitle.
+        // Get the `PES` packet containing the first chunk of our subtitle.
         let first: ps::PesPacket = try_iter!(self.pes_packets.next());
 
         // Fetch useful information from our first packet.
@@ -479,7 +354,7 @@ impl<'a> Iterator for SubtitlesInternal<'a> {
         let substream_id = first.pes_packet.substream_id;
 
         // Figure out how many total bytes we'll need to collect from one
-        // or more PES packets, and collect the first chunk into a buffer.
+        // or more `PES` packets, and collect the first chunk into a buffer.
         if first.pes_packet.data.len() < 2 {
             return Some(Err(VobSubError::PacketTooShort));
         }
@@ -517,83 +392,21 @@ impl<'a> Iterator for SubtitlesInternal<'a> {
             );
             sub_packet.truncate(wanted);
         }
+        Some(Ok((base_time, sub_packet)))
+    }
+}
+
+impl<D> Iterator for VobsubParser<'_, D> {
+    type Item = Result<(TimeSpan, VobSubIndexedImage), VobSubError>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        profiling::scope!("VobsubParser next");
+
+        let (base_time, sub_packet) = try_iter!(self.next_sub_packet());
+        let subtitle = subtitle::<(TimeSpan, VobSubIndexedImage), _>(&sub_packet, base_time);
 
         // Parse our subtitle buffer.
-        Some(subtitle(&sub_packet, base_time))
-    }
-}
-
-/// An iterator over subtitles.
-pub struct Subtitles<'a> {
-    internal: SubtitlesInternal<'a>,
-    prev: Option<Subtitle>,
-}
-
-impl<'a> Iterator for Subtitles<'a> {
-    type Item = Result<Subtitle, VobSubError>;
-
-    // This whole routine exists to make sure that `end_time` is set to a
-    // useful value even if the subtitles themselves didn't supply one.
-    // I'm not even sure this is valid, but it has been observed in the
-    // wild.
-    fn next(&mut self) -> Option<Self::Item> {
-        profiling::scope!("Subtitles next");
-
-        // If we don't currently have a previous subtitle, attempt to fetch
-        // one.
-        if self.prev.is_none() {
-            match self.internal.next() {
-                Some(Ok(sub)) => {
-                    self.prev = Some(sub);
-                }
-                other => return other,
-            }
-        }
-        debug_assert!(self.prev.is_some());
-
-        match self.internal.next() {
-            // We have a another subtitle!  We want to return `self.prev`
-            // and store the new subtitle as `self.prev`.
-            Some(Ok(curr)) => {
-                // `unwrap` is safe because of the invariant above.
-                let mut prev = self.prev.take().unwrap();
-                if prev.end_time.is_none() {
-                    // Our subtitle has no end time, so end it just before
-                    // the next subtitle.
-                    let new_end = curr.start_time - DEFAULT_SUBTITLE_SPACING;
-                    let alt_end = prev.start_time + DEFAULT_SUBTITLE_LENGTH;
-                    prev.end_time = Some(new_end.min(alt_end));
-                }
-                self.prev = Some(curr);
-                Some(Ok(prev))
-            }
-            // We encountered an error.  We could, I suppose, attempt to
-            // first return `self.prev` and save the error for next time,
-            // but that's too much trouble.
-            Some(Err(err)) => Some(Err(err)),
-            // The only subtitle left to return is `self.prev`.
-            None => {
-                self.prev.take().map(|mut sub| {
-                    if sub.end_time.is_none() {
-                        // Our subtitle has no end time, and it's the last
-                        // subtitle, so just pick something.
-                        sub.end_time = Some(sub.start_time + DEFAULT_SUBTITLE_LENGTH);
-                    }
-                    Ok(sub)
-                })
-            }
-        }
-    }
-}
-
-/// Return an iterator over the subtitles in this data stream.
-#[must_use]
-pub const fn subtitles(input: &[u8]) -> Subtitles {
-    Subtitles {
-        internal: SubtitlesInternal {
-            pes_packets: ps::pes_packets(input),
-        },
-        prev: None,
+        Some(subtitle)
     }
 }
 
@@ -670,18 +483,20 @@ mod tests {
         use std::fs;
         use std::io::prelude::*;
 
+        use crate::image::ImageArea;
+
         //let _ = env_logger::init();
 
         let mut f = fs::File::open("./fixtures/example.sub").unwrap();
         let mut buffer = vec![];
         f.read_to_end(&mut buffer).unwrap();
-        let mut subs = subtitles(&buffer);
-        let sub1 = subs.next().expect("missing sub 1").unwrap();
-        assert!(sub1.start_time - 49.4 < 0.1);
-        assert!(sub1.end_time.unwrap() - 50.9 < 0.1);
-        assert!(!sub1.force);
+        let mut subs = VobsubParser::<(TimeSpan, VobSubIndexedImage)>::new(&buffer);
+        let (time_span, img) = subs.next().expect("missing sub 1").unwrap();
+        assert!(time_span.start.to_secs() - 49.4 < 0.1);
+        assert!(time_span.end.to_secs() - 50.9 < 0.1);
+        //assert!(!sub1.force);
         assert_eq!(
-            sub1.area,
+            img.area(),
             Area::try_from(AreaValues {
                 x1: 750,
                 y1: 916,
@@ -690,8 +505,42 @@ mod tests {
             })
             .unwrap()
         );
-        assert_eq!(sub1.palette, [0, 3, 1, 0]);
-        assert_eq!(sub1.alpha, [15, 15, 15, 0]);
+        assert_eq!(*img.palette(), [0, 1, 3, 0]);
+        assert_eq!(*img.alpha(), [0, 15, 15, 15]);
+        subs.next().expect("missing sub 2").unwrap();
+        assert!(subs.next().is_none());
+    }
+
+    #[test]
+    fn parse_subtitles_times() {
+        //use env_logger;
+        use std::fs;
+        use std::io::prelude::*;
+
+        use crate::image::ImageArea;
+
+        //let _ = env_logger::init();
+
+        let mut f = fs::File::open("./fixtures/example.sub").unwrap();
+        let mut buffer = vec![];
+        f.read_to_end(&mut buffer).unwrap();
+        let mut subs = VobsubParser::<TimeSpan>::new(&buffer);
+        let (time_span, img) = subs.next().expect("missing sub 1").unwrap();
+        assert!(time_span.start.to_secs() - 49.4 < 0.1);
+        assert!(time_span.end.to_secs() - 50.9 < 0.1);
+        //assert!(!sub1.force);
+        assert_eq!(
+            img.area(),
+            Area::try_from(AreaValues {
+                x1: 750,
+                y1: 916,
+                x2: 1172,
+                y2: 966
+            })
+            .unwrap()
+        );
+        assert_eq!(*img.palette(), [0, 1, 3, 0]);
+        assert_eq!(*img.alpha(), [0, 15, 15, 15]);
         subs.next().expect("missing sub 2").unwrap();
         assert!(subs.next().is_none());
     }
@@ -702,7 +551,7 @@ mod tests {
         use idx::Index;
         //let _ = env_logger::init();
         let idx = Index::open("./fixtures/tiny.idx").unwrap();
-        let mut subs = idx.subtitles();
+        let mut subs = idx.subtitles::<TimeSpan>();
         subs.next().expect("missing sub").unwrap();
         assert!(subs.next().is_none());
     }
@@ -717,13 +566,13 @@ mod tests {
         // return the same subtitle data.
         let tiny = Index::open("./fixtures/tiny.idx")
             .unwrap()
-            .subtitles()
+            .subtitles::<TimeSpan>()
             .next()
             .unwrap()
             .unwrap();
         let split = Index::open("./fixtures/tiny-split.idx")
             .unwrap()
-            .subtitles()
+            .subtitles::<TimeSpan>()
             .next()
             .unwrap()
             .unwrap();
